//-----------------------------------------------------
//File: lcd_iic.c
//Auth: ELECTRONIC ASSEMBLY JM
//DATE: 08-25-2013
//-----------------------------------------------------

//---Includes---
#include "sfr_r825.h"
#include "define.h"
#include "lcd_iic.h"

#include "timer.h"

//--- module global varibles ---

unsigned char dh_function_set = 0x38;  //double hight font off default

//--- waitstates for testing ---
//use these macros to slow the communication down
#define WAITST 2
/*
#define Wait(x)
#define Wait(x) asm("nop")
*/
void Wait (unsigned char waitstates)
{
	unsigned char i=0;
	for (i=0; i<waitstates; i++)
	{
		asm("nop");
	}
}


//-----------------------------------------------------
//Func: initDispl()
//Desc: inits Display
//-----------------------------------------------------
void initDispl(void)
{
	unsigned char i;

//init IIC
	//Port init (dd -> datadirection register; 1 output, 0 input)
	ddSA0 = 1;
	SA0	  = 0; //--> Adress of DOGM is 0x78
	ddRES = 1;
	RES = 1;

	//IIC mode: (Init for R8C25)
	SDA = 1; //preset port level
	SCL = 1;
	ddSDA = 0; //SDA is input
	ddSCL = 0; //SCL is input

	mst_iccr1 = 0; 	//IIC receive
	iicsel = 1;     //set IIC Mode
	stop_icsr = 0;	//Clear the stop condition detection flag
	ice_iccr1 = 1;  //This module is enabled (Pins SCL and SDA are set to IIC port function)
	iccr1 = 0x80;	//Set tranfer clock (f1/256) and slave receive mode

	icmr = 0x00;	//No wait and transfer data with MSB first
	icier = 0x04;   //Transmit 0 at Acknowledgement in receive mode.
					//If receive acknowledge is set to 1, halt transfer.
					//Disable the stop condition detection interrupt request.
					//Disable the NACK receive interrupt and the arbitration lost interrupt request.
					//Disable the receive data full interrupt.
					//Disable the receive complete inteerrupt request.
					//Disable the receive data empty interrupt request.


	//Perform a display reset
	RES = 0;
	Delay(1);
	RES=1;
	Delay(1);

	//init Display
	WriteIns(0x3A);	//8-Bit data length extension Bit RE=1; REV=0
	WriteIns(0x09);	//4 line display
	WriteIns(0x06);	//Bottom view
	WriteIns(0x1E);	//Bias setting BS1=1
	WriteIns(0x39);	//8-Bit data length extension Bit RE=0; IS=1
	WriteIns(0x1B);	//BS0=1 -> Bias=1/6
	WriteIns(0x6E); //Devider on and set value
	WriteIns(0x56); //Booster on and set contrast (BB1=C5, DB0=C4)
	WriteIns(0x7A); //Set contrast (DB3-DB0=C3-C0)
	WriteIns(dh_function_set); //8-Bit data length extension Bit RE=0; IS=0

	ClrDisplay();
	DisplayOnOff(DISPLAY_ON | CURSOR_ON | BLINK_ON);
}

//-----------------------------------------------------
//Func: WriteChar(character)
//Desc: sends a single character to display
//-----------------------------------------------------
void WriteChar (char character)
{
	WriteData(character);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void WriteString(_far char * string)
{
	do
	{
		WriteData(*string++);
	}
	while(*string);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void DisplLines(unsigned char lines)
{
	if(lines == DOUBLEHEIGHT_OFF)
	{
		WriteIns(lines);
		dh_function_set = 0x38;
	}
	else
	{
		WriteIns(0x3C);
		WriteIns(0x3A);
		WriteIns(lines);
		WriteIns(0x3C);
		dh_function_set = 0x3C;
	}
	
}

//-----------------------------------------------------
//Func: SetPostion(postion)
//Desc: set postion
//-----------------------------------------------------
void SetPostion(char pos)
{
	WriteIns(LCD_HOME_L1+pos);
}

//-----------------------------------------------------
//Func: DisplayOnOff(control)
//Desc: use definitions of header file to set display
//-----------------------------------------------------
void DisplayOnOff(char data)
{
	WriteIns(0x08+data);
}

//-----------------------------------------------------
//Func: DefineCharacter(memory postion, character data)
//Desc: stores an own defined character
//-----------------------------------------------------
void DefineCharacter(unsigned char postion, unsigned char *data)
{
	unsigned char i=0;
	WriteIns(0x40+8*postion);

	for (i=0; i<8; i++)
	{
		WriteData(data[i]);
	}
	SetPostion(LINE1);
}
//-----------------------------------------------------
//Func: ClrDisplay
//Desc: Clears entire Display content and set home pos
//-----------------------------------------------------
void ClrDisplay(void)
{
	WriteIns(0x01);
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: SetContrast
//Desc: Sets contrast 0..63
//-----------------------------------------------------
void SetContrast(unsigned char contr)
{
	WriteIns(0x39);
	WriteIns(0x54 | (contr >> 4));
	WriteIns(0x70 | (contr & 0x0F));
	WriteIns(dh_function_set);

}

//-----------------------------------------------------
//Func: SetView
//Desc: view bottom view(0x06), top view (0x05)
//-----------------------------------------------------
void SetView(unsigned char view)
{
	WriteIns(0x3A);
	WriteIns(view);
	WriteIns(dh_function_set);
}

//-----------------------------------------------------
//Func: SetROM
//Desc: Changes the Rom code (ROMA=0x00, ROMB=0x04, ROMC=0x0C)
//---------------------------------------------------
void SetROM (unsigned char rom)
{
	WriteIns(0x2A);
	WriteIns(0x72);
	WriteData(rom);
	WriteIns(0x28);
}

//-----------------------------------------------------
//Func: WriteIns(instruction)
//Desc: sends instruction to display
//-----------------------------------------------------
static void WriteIns(char ins)
{
	CheckBusy();
	IIC_put(0x80, ins); //Controllbyte C0=0 & D/C = 0)

}

//-----------------------------------------------------
//Func: WriteData(data)
//Desc: sends data to display
//-----------------------------------------------------
static void WriteData(char data)
{
	CheckBusy();
	IIC_put(0x40, data); //Controllbyte C0=0 & D/C = 1)
}

//-----------------------------------------------------
//Func: CheckBusy()
//Desc: checks if display is idle
//-----------------------------------------------------
unsigned char CheckBusy(void)
{
	while(IIC_get() & 0x80)  //ckeck busyFlag
		;
}

//-----------------------------------------------------
//Func: IIC_put()
//Desc: waits till data is sent over the iic interface
//		returns 1 success, 0 error
//-----------------------------------------------------
static unsigned char IIC_put (unsigned char cb, unsigned char db)
{
	IIC_Start(); //Start IIC interface
	if (IIC_TxByte(ADR) == 0)  //adress
		return 0;
	if (IIC_TxByte(cb) == 0)	//control byte
		return 0;
	if (IIC_TxByte(db) == 0)	//data byte
		return 0;

	IIC_Stop(); //Stop IIC transfer
	return 1;
}

//-----------------------------------------------------
//Func: IIC_get()
//Desc: Gets one byte from display (busy flag check)
//-----------------------------------------------------
static unsigned char IIC_get(void)
{
	unsigned char ret;

	//Start reading busy flag
	IIC_Start();
	if (IIC_TxByte(ADR) == 0)
		return 0;
	if (IIC_TxByte(0x80) == 0) //control byte to display -> read busy flag and adress
		return 0;
	tend_icsr = 0;
	IIC_Stop();

	//Reading data
	IIC_Start();
	if (IIC_TxByte(ADR+1) == 0) //read from display
		return 0;

	tend_icsr = 0;	//Clear transmit end flag
	iccr1 &= 0xEF;	//Set Master receive mode
	tdre_icsr = 0;	//Clear transmit data empty flag
	ackbt_icier = 0; //When receiving, trasmit 0 at acknowledgement
	ret = icdrr;	//Start reception by dummy reading the ICDRR register
	while (rdrf_icsr == 0)
		;			//wait for receive buffer is full

	ackbt_icier = 1; //When receiving, trasmit 1 at acknowledgement, because only one byte receive
	rcvd_iccr1 = 1;
	Wait(100);
	while(rdrf_icsr == 0)  //wait for data receive
		;
	ret = icdrr;	//Received data read
	rcvd_iccr1 = 0;	//Continue to next operation after reading the ICDRR
	stop_icsr = 0;	//Clear the stop condition detection flag
	ackbt_icier = icdrr;	//When receiving transmit 0 at acknowledgement

	IIC_Stop();

	return ret;
}

//-----------------------------------------------------
//Func: IIC_Start()
//Desc: Issues start condition
//-----------------------------------------------------
static void IIC_Start(void)
{
	while (bbsy_iccr2 == 1)
		;				//Wait till bus is released

	stop_icsr = 0;	//Clear the stop condition detection flag
	iccr1 |= 0x30; //Master transmit mode
	iccr2 = 0xB0; //Generate the Start Codition
}

//-----------------------------------------------------
//Func: IIC_Start()
//Desc: Issues stop condition
//-----------------------------------------------------
static void IIC_Stop(void)
{
	tend_icsr = 0;

	iccr1 &= 0xEF; //Set receive mode
	iccr2 = 0x30; //Generate the stop condition
	tend_icsr = 0; //Clear transmit end flag
	nackf_icsr = 0; //Clear the no acknowledge detection flag
	while (stop_icsr == 0) //Ensure the STOP condition is detected
		;				   //wait until STOP flag is set
	stop_icsr = 0; //Clear STOP detection flag
}

//-----------------------------------------------------
//Func: IIC_TxByte()
//Desc: Sends one byte
//		returns 1 success, 0 error
//-----------------------------------------------------
static unsigned char IIC_TxByte(char data)
{
	icdrt = data; //Transmit data by writing into I2C bus data register
	while ( tdre_icsr == 0)  //Wait for data is transmitted to ICDRS
		;					 //wait while tranceive data register is not empty

	while (tend_icsr == 1)
		;					 //wait for the 9-th clock of transmit clock

	Wait(WAITST);
	if (nackf_icsr == 1)
		return 0;
	Wait(WAITST);
	return 1;
}

